from nltk.stem.porter import *
from nltk.tokenize.punkt import PunktWordTokenizer
from nltk.tokenize.punkt import PunktSentenceTokenizer
import solr
mainFolder = "../RelModel/"


def removeNonAscii(s): return "".join(filter(lambda x: ord(x)<128, s))

def processSolrHit(hit):
    for key in hit.keys():
        if key != 'score':
            hit[key] = [removeNonAscii(hit[key][0])]
    sents = (str(hit['text'][0])).split("newlinE")
    sents = [re.sub(r'\t+',' ',sent.replace("\'","\\\'").replace("\"","\\\"")).replace("\\","").encode('ASCII') for sent in sents]
    sents = sents[::2]
    return sents


def getDocumentSents(sentences):
    return ' '.join(sentences)



class parser():
    porter_stemmer = PorterStemmer()
    # len before adding= 671:
    stop_words = ['a', 'able', 'about', 'above', 'abst', 'accordance',
                  'according', 'accordingly', 'across', 'act', 'actually',
                  'added', 'adj', 'adopted', 'affected', 'affecting', 'affects',
                  'after', 'afterwards', 'again', 'against', 'ah', 'all',
                  'almost', 'alone', 'along', 'already', 'also', 'although',
                  'always', 'am', 'among', 'amongst', 'an', 'and', 'announce',
                  'another', 'any', 'anybody', 'anyhow', 'anymore', 'anyone',
                  'anything', 'anyway', 'anyways', 'anywhere', 'apparently',
                  'approximately', 'are', 'aren', 'arent', 'arise', 'around',
                  'as', 'aside', 'ask', 'asking', 'at', 'auth', 'available',
                  'away', 'awfully', 'b', 'back', 'be', 'became', 'because',
                  'become', 'becomes', 'becoming', 'been', 'before',
                  'beforehand', 'begin', 'beginning', 'beginnings', 'begins',
                  'behind', 'being', 'believe', 'below', 'beside', 'besides',
                  'between', 'beyond', 'biol', 'both', 'brief', 'briefly',
                  'but', 'by', 'c', 'ca', 'came', 'can', 'cannot', "can't",
                  'cause', 'causes', 'certain', 'certainly', 'co', 'com',
                  'come', 'comes', 'contain', 'containing', 'contains',
                  'could', "couldn't", 'd', 'date', 'did', "didn't", 'different',
                  'do', 'does', "doesn't", 'doing', 'done', "don't", 'down',
                  'downwards', 'due', 'during', 'e', 'each', 'ed', 'edu',
                  'effect', 'eg', 'eight', 'eighty', 'either', 'else',
                  'elsewhere', 'end', 'ending', 'enough', 'especially',
                  'et', 'et-al', 'etc', 'even', 'ever', 'every', 'everybody',
                  'everyone', 'everything', 'everywhere', 'ex', 'except', 'f',
                  'far', 'few', 'ff', 'fifth', 'first', 'five', 'fix',
                  'followed', 'following', 'follows', 'for', 'former',
                  'formerly', 'forth', 'found', 'four', 'from', 'further',
                  'furthermore', 'g', 'gave', 'get', 'gets', 'getting', 'give',
                  'given', 'gives', 'giving', 'go', 'goes', 'gone', 'got',
                  'gotten', 'h', 'had', 'happens', 'hardly', 'has', "hasn't",
                  'have', "haven't", 'having', 'he', 'hed', 'hence', 'her',
                  'here', 'hereafter', 'hereby', 'herein', 'heres', 'hereupon',
                  'hers', 'herself', 'hes', 'hi', 'hid', 'him', 'himself',
                  'his', 'hither', 'home', 'how', 'howbeit', 'however',
                  'hundred', 'i', 'id', 'ie', 'if', "i'll", 'im', 'immediate',
                  'immediately', 'importance', 'important', 'in', 'inc',
                  'indeed', 'index', 'information', 'instead', 'into',
                  'invention', 'inward', 'is', "isn't", 'it', 'itd', "it'll",
                  'its', 'itself', "i've", 'j', 'just', 'k', 'keep', 'keeps',
                  'kept', 'keys', 'kg', 'km', 'know', 'known', 'knows', 'l',
                  'largely', 'last', 'lately', 'later', 'latter', 'latterly',
                  'least', 'less', 'lest', 'let', 'lets', 'like', 'liked',
                  'likely', 'line', 'little', "'ll", 'look', 'looking', 'looks',
                  'ltd', 'm', 'made', 'mainly', 'make', 'makes', 'many', 'may',
                  'maybe', 'me', 'mean', 'means', 'meantime', 'meanwhile',
                  'merely', 'mg', 'might', 'million', 'miss', 'ml', 'more',
                  'moreover', 'most', 'mostly', 'mr', 'mrs', 'much', 'mug',
                  'must', 'my', 'myself', 'n', 'na', 'name', 'namely', 'nay',
                  'nd', 'near', 'nearly', 'necessarily', 'necessary', 'need',
                  'needs', 'neither', 'never', 'nevertheless', 'new', 'next',
                  'nine', 'ninety', 'no', 'nobody', 'non', 'none',
                  'nonetheless', 'noone', 'nor', 'normally', 'nos', 'not',
                  'noted', 'nothing', 'now', 'nowhere', 'o', 'obtain',
                  'obtained', 'obviously', 'of', 'off', 'often', 'oh', 'ok',
                  'okay', 'old', 'omitted', 'on', 'once', 'one', 'ones', 'only',
                  'onto', 'or', 'ord', 'other', 'others', 'otherwise', 'ought',
                  'our', 'ours', 'ourselves', 'out', 'outside', 'over',
                  'overall', 'owing', 'own', 'p', 'page', 'pages', 'part',
                  'particular', 'particularly', 'past', 'per', 'perhaps',
                  'placed', 'please', 'plus', 'poorly', 'possible', 'possibly',
                  'potentially', 'pp', 'predominantly', 'present', 'previously',
                  'primarily', 'probably', 'promptly', 'proud', 'provides',
                  'put', 'q', 'que', 'quickly', 'quite', 'qv', 'r', 'ran',
                  'rather', 'rd', 're', 'readily', 'really', 'recent',
                  'recently', 'ref', 'refs', 'regarding', 'regardless',
                  'regards', 'related', 'relatively', 'research',
                  'respectively', 'resulted', 'resulting', 'results', 'right',
                  'run', 's', 'said', 'same', 'saw', 'say', 'saying', 'says',
                  'sec', 'section', 'see', 'seeing', 'seem', 'seemed',
                  'seeming', 'seems', 'seen', 'self', 'selves', 'sent', 'seven',
                  'several', 'shall', 'she', 'shed', "she'll", 'shes', 'should',
                  "shouldn't", 'show', 'showed', 'shown', 'showns', 'shows',
                  'significant', 'significantly', 'similar', 'similarly',
                  'since', 'six', 'slightly', 'so', 'some', 'somebody',
                  'somehow', 'someone', 'somethan', 'something', 'sometime',
                  'sometimes', 'somewhat', 'somewhere', 'soon', 'sorry',
                  'specifically', 'specified', 'specify', 'specifying',
                  'state', 'states', 'still', 'stop', 'strongly', 'sub',
                  'substantially', 'successfully', 'such', 'sufficiently',
                  'suggest', 'sup', 'sure', 't', 'take', 'taken', 'taking',
                  'tell', 'tends', 'th', 'than', 'thank', 'thanks', 'thanx',
                  'that', "that'll", 'thats', "that've", 'the', 'their',
                  'theirs', 'them', 'themselves', 'then', 'thence', 'there',
                  'thereafter', 'thereby', 'thered', 'therefore', 'therein',
                  "there'll", 'thereof', 'therere', 'theres', 'thereto',
                  'thereupon', "there've", 'these', 'they', 'theyd', "they'll",
                  'theyre', "they've", 'think', 'this', 'those', 'thou',
                  'though', 'thoughh', 'thousand', 'throug', 'through',
                  'throughout', 'thru', 'thus', 'til', 'tip', 'to', 'together',
                  'too', 'took', 'toward', 'towards', 'tried', 'tries', 'truly',
                  'try', 'trying', 'ts', 'twice', 'two', 'u', 'un', 'under',
                  'unfortunately', 'unless', 'unlike', 'unlikely', 'until',
                  'unto', 'up', 'upon', 'ups', 'us', 'use', 'used', 'useful',
                  'usefully', 'usefulness', 'uses', 'using', 'usually', 'v',
                  'value', 'various', "'ve", 'very', 'via', 'viz', 'vol',
                  'vols', 'vs', 'w', 'want', 'wants', 'was', "wasn't", 'way',
                  'we', 'wed', 'welcome', "we'll", 'went', 'were', "weren't",
                  "we've", 'what', 'whatever', "what'll", 'whats', 'when',
                  'whence', 'whenever', 'where', 'whereafter', 'whereas',
                  'whereby', 'wherein', 'wheres', 'whereupon', 'wherever',
                  'whether', 'which', 'while', 'whim', 'whither', 'who', 'whod',
                  'whoever', 'whole', "who'll", 'whom', 'whomever', 'whos',
                  'whose', 'why', 'widely', 'willing', 'wish', 'with', 'within',
                  'without', "won't", 'words', 'world', 'would', "wouldn't",
                  'www', 'x', 'y', 'yes', 'yet', 'you', 'youd', "you'll",
                  'your', 'youre', 'yours', 'yourself', 'yourselves', "you've",
                  'z', 'zero',
                  #punctuation marks:
                  '`', "'", '(', ')', ',', '_', ';', ':', '~', '&', '-', '--',
                  '$', '^', '*', #maybe more
                  #added due to word tokenizer:
                  "'s", "'ll", "'t", "'ve", "'m",
                  'doesn', 'don', 'hasn', 'haven', 'isn', 'wasn', 'won', 'weren',
                  'wouldn', 'didn', 'shouldn', 'couldn']
    
    def __init__(self, mainFolder, topic, query):
        self._mainfolder = mainFolder
        self._query = {'topic': topic,'query':query}
        self._wordTokenizer = PunktWordTokenizer()
        self._sentTokenzier = PunktSentenceTokenizer()
        self._sentences = None


    def getTextQuery(self, query):
        query = query['query'].strip()
        cutoff = 10
        solr_servers = {}
        solr_servers['asthma'] = 'http://www.cs.bgu.ac.il/~talbau/solr/'
        solr_servers['cancer'] = 'http://www.cs.bgu.ac.il/~talbau/solrcancer/'
        solr_servers['alz'] = 'http://www.cs.bgu.ac.il/~talbau/solralz/'
        solr_servers['obese'] = 'http://www.cs.bgu.ac.il/~talbau/solrobese/'
        s = solr.SolrConnection(solr_servers[self._query['topic']])

        docs = []
        start = 0
        doccount = 0
        response = s.query(query,start = start)
        while response:
            for hit in response.results:
                doccount += 1
                docs += processSolrHit(hit)
                if doccount > cutoff:
                    break
            if doccount > cutoff:
                break
            start += 10
            response = s.query(query,start = start)
        sents = getDocumentSents(docs)
        return sents

    # private:
    def __initDocumentSent(self):
        text = ""
        text+= self.getTextQuery(self._query)
        self._sentences =self._sentTokenzier.tokenize(text)
    
    #returns a list of 2 elements:
    #   1. list of ORIGINAL sentences in the documents as string.
    #   2. list of sentences as list of normalized words.
    def getDocumentSents(self):
        if not self._sentences:
            self.__initDocumentSent()
        normalized_sents=[]
        for sent in self._sentences:
            norm_words = []
            #sent[:-1] in order to remove the dot at the end of the sentence:
            for word in self._wordTokenizer.tokenize(sent[:-1]):
                l_word = word.lower()
                if l_word not in self.stop_words:
                    norm_words += [self.porter_stemmer.stem(l_word)]
            normalized_sents += [norm_words]
        return [self._sentences, normalized_sents]

    def get_num_of_files(self):
        return len(self._query)
    #returns: the number of docs word w is in for every word in the self.docs
    def getDocCountForWords(self):
        topicDocs = self._query
        res=dict()
        text = ""

        text= self.getTextQuery(self._query)
        words=self._wordTokenizer.tokenize(text)
        temp=dict()
        for word in words:
            if (temp.get(word,0)==0):
                res[word]=res.get(word,0)+1
                temp[word]=1
        return res
    
#### test ####
 
#p = parser(mainFolder,indexfile,test_topic)
#print p.getDocumentSents()[-1]
#print 'end'

